
--[[ EXAMPLE : CUBE

Usage:
  This program draws a simple cube on screen.
  You may rotate it with the arrow keys.
  Press the Break button to stop.
  
Illustrates:
  * tables and matrices
  * "draw" package
  
Training:
  * Change the PERSP value and the position
    of the camera and see the effect
  * Try to modify the cube,
    and to create some other 3D objects
  * Add some new item transformations
  * Modify the program to let the user move
    the camera with the keyboard 
  * What happens if a point is "behind" the camera?
    ( i.e. pt[3] <= cam[3] )
    Could you correct this effect?
]]


-- REQUIRED LIBS AND PACKAGES

require("draw")


-- CONSTANTS AND FUNCTIONS

  -- distance between screen and camera
PERSP = 400

  -- camera position
cam = { 0, 0, -200 }

  -- draw a 3D item on screen
function drawItem(item)
  local i=1
  local lines = item.lines
  local points = item.points
  local n = #lines-1

    -- draw each line
  while i<=n do
    local l1 = lines[i]
    local l2 = lines[i+1]
   
    if l2==0 then i = i+2
    elseif l1==0 then i = i+1
    else
      local p1 = points[l1]
      local p2 = points[l2]
     
      local x1 = 80  + cam[1] + p1[1] * PERSP / (p1[3]-cam[3])
      local y1 = 100 + cam[2] + p1[2] * PERSP / (p1[3]-cam[3])
      local x2 = 80  + cam[1] + p2[1] * PERSP / (p2[3]-cam[3])
      local y2 = 100 + cam[2] + p2[2] * PERSP / (p2[3]-cam[3])
     
      draw.line(x1,y1,x2,y2)
      i = i+1
    end
  end
end

  -- matrix product
function prodmat(points, t)
  for i, pt in ipairs(points) do
    points[i] = 
      { pt[1] * t[1][1] + pt[2] * t[1][2] + pt[3] * t[1][3],
        pt[1] * t[2][1] + pt[2] * t[2][2] + pt[3] * t[2][3],
        pt[1] * t[3][1] + pt[2] * t[3][2] + pt[3] * t[3][3] }
  end
end

  -- rotate an item (X axis)
function rotateItemX(item, r) 
  local t = 
    { { 1, 0, 0 },
      { 0, math.cos(r), - math.sin(r) },
      { 0, math.sin(r), math.cos(r) } }
  prodmat(item.points, t)
end

  -- rotate an item (Y axis)
function rotateItemY(item, r)
  local t = 
  { { math.cos(r), 0, math.sin(r) },
    { 0, 1, 0 },
    { - math.sin(r), 0, math.cos(r) } }
  prodmat(item.points, t)
end


-- THE PROGRAM STARTS HERE

  -- This is a simple cube.
  -- A 3D item is a table with 2 fields:
  -- * 'points' is a list with each item's points
  -- * 'lines' is a list with the connections
  -- between the points. 2 successive numbers
  -- indicate a line between the points with
  -- these numbers. A zero value breaks the
  -- sequence.
  
C = 10
anItem = {

  points = { 
    {C, -C, C},   -- first point (P1)
    {C, C, C},    -- second point (P2)
    {C, -C, -C},  -- P3
    {C, C, -C},   -- P4
    {-C, -C, C},  -- P5
    {-C, C, C},   -- P6
    {-C, -C, -C}, -- P7
    {-C, C, -C}   -- P8
  },
   
  lines = { 
      -- P1-P2, P2-P4, P4-P3, ...
    1,2,4,3,1,5,6,8,7,5, 0,
      -- P2-P6
    2,6, 0, 
      -- P4-P8
    4,8, 0, 
    3,7, 0, 
    1,4, 0, 
    2,3 
  }
}
  
keypad(0)
showgraph()

draw.onbuffer()

draw.clear()
drawItem(anItem)
draw.update()

while 1 do
  local changed = false
      
	-- rotate the item
  if testkey(K_RIGHT) then 
	rotateItemY(anItem, -0.3)
	changed = true
  elseif testkey(K_LEFT) then 
	rotateItemY(anItem, 0.3)
	changed = true
  end
  
  if testkey(K_DOWN) then 
	rotateItemX(anItem, 0.3)
	changed = true
  elseif testkey(K_UP) then 
	rotateItemX(anItem, -0.3)
	changed = true
  end

  if changed then    -- and redraw the screen
    draw.clear()
    drawItem(anItem)
    draw.update()
  end
end
  
  
