----------------------------------------------------------------
-- EXAMPLE 3
--
-- This program shows how you can create a custom widget
-- and use it in a simple window
----------------------------------------------------------------

require("ui", "draw")

-- 1. We create a custom widget with some 
-- new functions.
-- This might be written in a library instead;
-- For this example, the two parts have been
-- kept in one file.


-- Functions to be called by the widget
-- when the pen is used.
-- Each scribble widget will share the same
-- function; this uses less memory 
-- than copying the function for each created widget.
function scribble_pendown(wid, x, y)
    local old = draw.onwhat()
    local th = wid:getthickness()
    
    draw.onpict( wid:getpict() )
    
    draw.point(x, y, COL_BLACK, wid.thickness)
    wid:update()

    wid.oldx = x
    wid.oldy = y
    
    draw.onpict(old)
end


function scribble_penmove(wid, x, y)
    local old = draw.onwhat()
    local th = wid:getthickness()

    draw.onpict( wid:getpict() )
    
    if wid.oldx == nil then
      draw.point(x, y, COL_BLACK, th)
    else
      draw.line(x, y, wid.oldx, wid.oldy, COL_BLACK, th)
    end
    wid:update()
    
    wid.oldx = x
    wid.oldy = y

    draw.onpict(old)
end


function scribble_penup(wid, w, y)
    wid.oldx, wid.oldy = nil
end


function scribble_clear(wid)
    local old = draw.onwhat()
    draw.onpict( wid:getpict() )
    draw.clear()
    wid:update()
    draw.onpict(old)
end

-- Create the new widget and define 
-- its members, functions and events
function scribbleWidget(args)
  local wid = ui.widget(args)
  
  wid._pendown = scribble_pendown
  wid._penmove = scribble_penmove
  wid._penup = scribble_penup
  
  local thickness = args.thickness or 2
  
  -- "thickness" is an upvalue
  function wid:setthickness(th)
    thickness = th
  end
  
  function wid:getthickness()
    return thickness
  end
  
  wid.clear = scribble_clear

  return wid
end

-- If the previous code is in a library, add
-- the following line:
--export{scribbleWidget=scribbleWidget}
-- Then the script that called the library
-- will be able to use scribbleWidget().

-- 2. Now we write a script to make
-- a program that uses the new widget.

-- Create a new window
win = ui.window{
  name="test", 
  x=0, y=10, width=160, height=130}

-- Give it a Scribble Widget
win.scribble = scribbleWidget{
  x=10, y=16, width=140, height=80,
  frame="thick", parent=win, thickness=2}

-- Write some text inside the window
draw.onpict( win:getpict() )
draw.text(58,3,"Scribble")
draw.text(10,108,"Size:")

-- Create a spinbox to control
-- the scribbleWidget's pen size
win.sizeSpinbox = ui.spinbox{
  x=43, y=104, width=25,
  value=2, min=1, max=5, parent=win}
  
function win.sizeSpinbox:_valuechanged(val)
  win.scribble:setthickness(val)
end

-- Create a button to clear the widget
win.ClearBtn = ui.button{
  x=80, y=106, text="Clear", parent=win}
  
function win.ClearBtn:_clicked()
  win.scribble:clear()
end

-- And another button to quit
win.QuitBtn = ui.button{
  x=125, y=106, text="Quit", parent=win}
  
function win.QuitBtn:_clicked()
  ui.stop()
end

-- Make the window visible
-- and start the UI application
win:show(true)
ui.start()
